function model = team_training_code(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find text files


patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes_murmur={};
classes_outcome={};
for j=1:num_patient_files

    current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
    classes_murmur=unique([classes_murmur current_class_murmur]);

    current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
    classes_outcome=unique([classes_outcome current_class_outcome]);

end

classes_murmur=sort(classes_murmur);
num_classes_murmur=length(classes_murmur);

classes_outcome=sort(classes_outcome);
num_classes_outcome=length(classes_outcome);

% Extracting features and labels
disp('Extracting features and labels...')

features=[];
labels_murmur=categorical;
labels_outcome=categorical;

FragIl=4000;
FragStep=2000;
Fs=4000;
PacBeg=true;
[minfreq,maxfreq] = cwtfreqbounds(4000,Fs);
vMurMur=[];


for j=1:num_patient_files
    labels_murmur(j)=get_class_murmur(fullfile(input_directory,patient_files{j}));
    labels_outcome(j)=get_class_outcome(fullfile(input_directory,patient_files{j}));
    if isequal(cellstr(labels_murmur(j)),{'Absent'})
                MurMur=0;
                KCikSk=5;
    elseif isequal(cellstr(labels_murmur(j)),{'Present'})
                MurMur=1;
                KCikSk=20;
    else
                MurMur=2;
                KCikSk=80;
    end
    if isequal(cellstr(labels_outcome(j)),{'Normal'})
                MurMur_outcome=0;         
    else
                MurMur_outcome=1;
               
    end
    pacientas=patient_files{j};
    fprintf('%d/%d \n',j,num_patient_files)

    current_header=get_header(fullfile(input_directory,patient_files{j}));
    current_recordings=load_recordings(input_directory,current_header);

    current_features=get_features(current_header, current_recordings);
  for i=1:length(current_recordings)
        signal=current_recordings{1,i};
        kp=round(length(signal)/FragStep)-3;
        wt=[];
        ent=[];
        for x=1:kp
            pra=round(1+(x-1)*FragStep);
            pbg=FragIl+(x-1)*FragStep;
            [wttt,f]=cwt(signal(pra:pbg,1),Fs,'FrequencyLimits',[minfreq maxfreq]);
            wt(:,:,x)=imresize(abs(wttt(:,1:FragIl)),[91 91]);
            ent(x,1)=entropy(wt(:,:,x));
            ent(x,2)=x;
        end
        if kp >KCikSk
           kkp=KCikSk;
        else
           kkp=kp;
        end 
        ents=sortrows(ent,1);
        ents(ents(:,1)==0,:)=[];
        mwt=wt(:,:,ents(1:kkp,2));
        if PacBeg
                features=mwt;
                vMurMur(:,1)=MurMur*ones(kkp,1);
                vMurMur(:,2)=str2double(pacientas(1:length(pacientas)-4))*ones(kkp,1);
                vMurMur(:,3)=MurMur_outcome*ones(kkp,1);
                PacBeg=false;
            else
                [vn,vm,vl]=size(features);
                vMurMur(vl+1:vl+kkp,1)=MurMur*ones(kkp,1);
                vMurMur(vl+1:vl+kkp,2)=str2double(pacientas(1:length(pacientas)-4))*ones(kkp,1);
                vMurMur(vl+1:vl+kkp,3)=MurMur_outcome*ones(kkp,1);
                features(:,:,vl+1:vl+kkp)=mwt;
        end
    end
end
    

filename ='D:\Physionet2022\Mano_edited\test5.mat';
save(filename,'features','vMurMur')
%% train RF

disp('Training the model...')


kiek=find(vMurMur(:,1)==1);
reikalingi1=find(vMurMur(:,1)==1,length(kiek));
reikalingi2=find(vMurMur(:,1)==0,length(kiek));
reikalingi=[reikalingi1;reikalingi2];

YTrain=categorical(vMurMur(reikalingi,1));
XTrain(:,:,1,:)=features(:,:,reikalingi);

idx = randperm(size(XTrain,4),round(length(features)*0.2));
XValidation = XTrain(:,:,:,idx);
XTrain(:,:,:,idx) = []; 
YValidation = YTrain(idx);
YTrain(idx) = [];

imageAugmenter = imageDataAugmenter( ...
    'RandRotation',[0,0], ...
    'RandXTranslation',[0 0], ...
    'RandYTranslation',[0 0]);

imageSize = [91 91 1];
augimds = augmentedImageDatastore(imageSize,XTrain,YTrain,'DataAugmentation',imageAugmenter);

layers = [
    imageInputLayer(imageSize)
   
    convolution2dLayer(3,8,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2)  
   
    averagePooling2dLayer(5,'Stride',2)
   
    convolution2dLayer(3,16,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2)   
   
    averagePooling2dLayer(2,'Stride',2)
   
    convolution2dLayer(3,32,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2)   
   
    fullyConnectedLayer(2)
    softmaxLayer
    classificationLayer];


opts = trainingOptions('sgdm', ...
    'MaxEpochs',15, ...
    'Shuffle','every-epoch', ...
    'Plots','none', ...
    'Verbose',false, ...
    'ValidationData',{XValidation,YValidation});

model_murmur = trainNetwork(augimds,layers,opts);

kiek=find(vMurMur(:,3)==0);
reikalingi1=find(vMurMur(:,1)==1 & vMurMur(:,3)==1,length(kiek));
reikalingi2=find(vMurMur(:,1)==0 & vMurMur(:,3)==0,length(kiek));
reikalingi=[reikalingi1;reikalingi2];



YTrain2=categorical(vMurMur(reikalingi,3));
XTrain2(:,:,1,:)=features(:,:,reikalingi);

idx = randperm(size(XTrain2,4),round(length(reikalingi)*0.2));
XValidation2 = XTrain2(:,:,:,idx);
XTrain2(:,:,:,idx) = []; 
YValidation2 = YTrain2(idx);
YTrain2(idx) = [];

layers2 = [
    imageInputLayer(imageSize)
   
    convolution2dLayer(3,8,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2.7)  
   
    averagePooling2dLayer(5,'Stride',2)
   
    convolution2dLayer(3,16,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2.7)   
   
    averagePooling2dLayer(2,'Stride',2)
   
    convolution2dLayer(3,32,'Padding','same')
    batchNormalizationLayer
    clippedReluLayer(2.7)   
   
    fullyConnectedLayer(2)
    softmaxLayer
    classificationLayer];

opts2 = trainingOptions('sgdm', ...
    'MaxEpochs',15, ...
    'Shuffle','every-epoch', ...
    'Plots','none', ...
    'Verbose',false, ...
    'ValidationData',{XValidation2,YValidation2});


imageSize = [91 91 1];
augimds2 = augmentedImageDatastore(imageSize,XTrain2,YTrain2,'DataAugmentation',imageAugmenter);

model_outcome=trainNetwork(augimds2,layers2,opts2);



save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory);

disp('Done.')

end

function save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur','classes_murmur','model_outcome','classes_outcome','-v7.3');

disp('Done.')
end

function class=get_class_murmur(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function class=get_class_outcome(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Outcome'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)

    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));

end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1

    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};

end

end
